create database 'C:\Interbase\DatabaseProjects\Bibliotek\Bibliotek.gdb'
user 'SYSDBA' password 'masterkey';

/* Domain definitioner */
create domain NavnDomain as VarChar(30);
create domain AdresseDomain as VarChar(30);
create domain PostNrByDomain as VarChar(30);
create domain TlfDomain as Char(8);
create domain CprDomain as Char(11);
create domain ISBNDomain as VarChar(14);
create domain AntalDomain as SmallInt check (VALUE>=0);

/* Tabel-Definitioner */
create table Laaner (
        Laaner_ID CprDomain not null,
        Navn NavnDomain not null,
        Adresse AdresseDomain not null,
        PostNrBy PostNrByDomain not null,
        Tlf TlfDomain,
        Reg_Dato DATE default 'NOW',
    primary key (Laaner_ID)
);

create table Bog (
        ISBN ISBNDomain not null,
        Titel NavnDomain not null,
        Forfatter NavnDomain not null,
        DecKlasse VarChar(8) not null,
        Antal AntalDomain,
        Reg_Dato DATE default 'NOW',
        Annotation VarChar(512),
    primary key (ISBN)
);

create table Udlaan (
        ISBN ISBNDomain not null,
        Laaner_ID CprDomain not null,
        Dato DATE default 'NOW',
/*        Dage computed by ('NOW' - Dato),*/
    primary key (ISBN,Laaner_ID)
);

/* Fremmed ngler definitioner */

alter table Udlaan add
  constraint ISBN_Findes_Ikke
  foreign key (ISBN) REFERENCES Bog (ISBN);

alter table Udlaan add
  constraint Laaner_Findes_Ikke
  foreign key (Laaner_ID) REFERENCES Laaner (Laaner_ID);

/* Sekundre (sge) Index */

create ascending index
AdresseIndex on Laaner (Adresse);

create ascending index
NavnIndex on Laaner (Navn);

create ascending index
TlfIndex on Laaner (Tlf);

create ascending index
ForfatterIndex on Bog (Forfatter);

create ascending index
TitelIndex on Bog (Titel);

create ascending index
DatoIndex on UdLaan (Dato);

create ascending index
ISBNIndex on Udlaan (ISBN);

/* Trigger-definitioner til at bevare Database reference integritet og konsistens */
/* En Trigger er en operation, der automatisk udlses af en bestemt hndelse.
   for eksempel, at der slettes en rkke (tuple) i en tabel */

set TERM  ^;

/* Ved_Slet_Bog  er en Trigger, som srger for at udlnsregistreringen af en bog,
   bliver slettet, hvis bogen slettes af samlingen. Ndvendigt af hensyn til
   reference integritet */

create trigger Ved_Slet_Bog for Bog
inactive before delete as
begin
  delete from Udlaan where Bog.ISBN=Udlaan.ISBN;
end^

/* Ved_Slet_Laaner  er en Trigger, som srger for at en Lners udln,
   bliver slettet, hvis Lneren slettes af lnerkartoteket. Ndvendigt af hensyn
   til reference integritet */

create trigger Ved_Slet_Laaner for Laaner
inactive before delete as
begin
  delete from Udlaan where Laaner.Laaner_ID=Udlaan.Laaner_ID;
end^

commit;


