import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;

public class KugleAnimationMenu {

  int[] x = new int[5];
  int[] y = new int[5];
  int[] vx = new int[5];
  int[] vy = new int[5];

  byte antal=1;

  final static int WIDTH=500;
  final static int HEIGHT=400;
  final static int R=40;
  final static int DT=4;
  final static int V0=10;
  final static int DELAY=50;

  byte[] farve = new byte[5];

  public void sletKugler(DrawingCanvas canvas) {
    Image offScreenImage = canvas.getOffscreenImage();
    Graphics imageGraphics = offScreenImage.getGraphics();
    Graphics screen = canvas.getGraphics();

    imageGraphics.setColor(Color.lightGray);
    for (int k=0; k<(int)antal; k++) {
      imageGraphics.fillOval(x[k]-R/2,y[k]-R/2,R,R);
      canvas.paint(screen);
    }
  }


  public void tegnKugler(DrawingCanvas canvas) {
    Image offScreenImage = canvas.getOffscreenImage();
    Graphics imageGraphics = offScreenImage.getGraphics();
    Graphics screen = canvas.getGraphics();

    for (int k=0; k<(int)antal; k++) {
      switch (farve[k]) {
        case 0: imageGraphics.setColor(Color.black); break;
        case 4: imageGraphics.setColor(Color.white); break;
        case 1: imageGraphics.setColor(Color.red); break;
        case 2: imageGraphics.setColor(Color.green); break;
        case 3: imageGraphics.setColor(Color.blue);
      }

      imageGraphics.fillOval(x[k]-R/2,y[k]-R/2,R,R);
      canvas.paint(screen);
    }
 }

  public void flytKugler() {
    int R2=R/2;
    for (int k=0; k<(int)antal; k++) {
      x[k]=x[k]+vx[k]*DT;
      y[k]=y[k]+vy[k]*DT;
      if ((x[k]+R2>=WIDTH)||(x[k]-R2<=0)) vx[k]=-vx[k];
      if ((y[k]+R2>=HEIGHT)||(y[k]-R2<=0)) vy[k]=-vy[k];
    }
  }

  public static void main(String[] args) {
    KugleAnimationMenu kugler = new KugleAnimationMenu();

    JFrame frame = new JFrame("Kugler i en Box");

    Container pane = frame.getContentPane();

    DrawingCanvas canvas = new DrawingCanvas(kugler.WIDTH,kugler.HEIGHT);

    pane.add(canvas);

    Timer timer = new Timer(DELAY, new Animate(canvas, kugler));

    JMenuBar menuBar = new JMenuBar();

    JMenu startMenu = new JMenu("Spil");
    startMenu.setMnemonic('I');

    JMenu farveMenu = new JMenu("Farve");
    farveMenu.setMnemonic('F');

    JMenu nyKugleMenu = new JMenu("Kugle");
    nyKugleMenu.setMnemonic('K');

    menuBar.add(startMenu);
    menuBar.add(farveMenu);
    menuBar.add(nyKugleMenu);

    frame.setJMenuBar(menuBar);

    JMenuItem start = new JMenuItem("Start",'S');
    JMenuItem stop = new JMenuItem("Stop",'P');
    JMenuItem luk = new JMenuItem("Luk",'L');
    startMenu.add(start);
    startMenu.add(stop);
    startMenu.add(luk);
    luk.addActionListener(new Luk());
    start.addActionListener(new Start(canvas, kugler, timer));
    stop.addActionListener(new Stop(timer));
    JMenuItem sort = new JMenuItem("Sort",'S');
    JMenuItem roed = new JMenuItem("Rd",'R');
    JMenuItem groen = new JMenuItem("Grn",'G');
    JMenuItem blaa = new JMenuItem("Bl",'B');
    JMenuItem hvid = new JMenuItem("Hvid",'H');
    farveMenu.add(sort);
    farveMenu.add(roed);
    farveMenu.add(groen);
    farveMenu.add(blaa);
    farveMenu.add(hvid);
    sort.addActionListener(new Sort(kugler));
    roed.addActionListener(new Roed(kugler));
    groen.addActionListener(new Groen(kugler));
    blaa.addActionListener(new Blaa(kugler));
    hvid.addActionListener(new Hvid(kugler));

    JMenuItem nyKugle = new JMenuItem("Ny Kugle",'N');
    JMenuItem fjernKugle = new JMenuItem("Fjern Kugle",'F');
    nyKugleMenu.add(nyKugle);
    nyKugleMenu.add(fjernKugle);
    nyKugle.addActionListener(new NyKugle(kugler));
    fjernKugle.addActionListener(new FjernKugle(kugler));

    MenuListener menuListener = new MainMenuListener(timer);
    startMenu.addMenuListener(menuListener);
    farveMenu.addMenuListener(menuListener);
    nyKugleMenu.addMenuListener(menuListener);

    frame.pack();
    frame.show();

    Image offScreenImage = canvas.getOffscreenImage();
    Graphics imageGraphics = offScreenImage.getGraphics();

    imageGraphics.setColor(Color.lightGray);
    imageGraphics.fillRect(0,0,WIDTH,HEIGHT);
  }
}

class Animate implements ActionListener {
  KugleAnimationMenu kugler;
  DrawingCanvas canvas;

  Animate(DrawingCanvas canvas, KugleAnimationMenu kugler) {
    this.kugler = kugler;
    this.canvas = canvas;
  }

  public void actionPerformed(ActionEvent e) {
    kugler.sletKugler(canvas);
    kugler.flytKugler();
    kugler.tegnKugler(canvas);
  }
}

class NyKugle implements ActionListener {
  KugleAnimationMenu kugler;
  NyKugle(KugleAnimationMenu kugler) {
    this.kugler = kugler;
  }
  public void actionPerformed(ActionEvent e) {
    byte k=kugler.antal;
    double w=(double)kugler.WIDTH;
    double h=(double)kugler.HEIGHT;
    double v0=(double)kugler.V0;
    double startVinkel=(Math.random()*2-1)*Math.PI;
    kugler.x[k]=(int)(Math.random()*0.8*w+0.1*w);
    kugler.y[k]=(int)(Math.random()*0.8*h+0.1*h);
    kugler.vx[k]=(int) (v0*Math.cos(startVinkel));
    kugler.vy[k]=(int) (v0*Math.sin(startVinkel));
    kugler.antal++;
  }
}

class FjernKugle implements ActionListener {
  KugleAnimationMenu kugler;
  FjernKugle(KugleAnimationMenu kugler) {
    this.kugler = kugler;
  }
  public void actionPerformed(ActionEvent e) {
    kugler.antal--;
  }
}

class Luk implements ActionListener {
  public void actionPerformed(ActionEvent e) {
    System.exit(0);
  }
}

class Stop implements ActionListener {
  Timer timer;
  Stop(Timer timer) {
    this.timer = timer;
  }
  public void actionPerformed(ActionEvent e) {
    timer.stop();
  }
}

class Start implements ActionListener {
  KugleAnimationMenu kugler;
  DrawingCanvas canvas;
  Timer timer;

  Start(DrawingCanvas canvas, KugleAnimationMenu kugler, Timer timer) {
    this.kugler = kugler;
    this.canvas = canvas;
    this.timer = timer;
  }

  public void actionPerformed(ActionEvent e) {
    kugler.antal=1;
    double w=(double)kugler.WIDTH;
    double h=(double)kugler.HEIGHT;
    double v0=(double)kugler.V0;
    double startVinkel=(Math.random()*2-1)*Math.PI;
    kugler.x[0]= (int)w/2;
    kugler.y[0]= (int)h/2;
    kugler.vx[0]=(int) (v0*Math.cos(startVinkel));
    kugler.vy[0]=(int) (v0*Math.sin(startVinkel));
    kugler.farve[0] = 1;
    timer.start();
  }
}

class Sort implements ActionListener {
  KugleAnimationMenu kugler;
  Sort(KugleAnimationMenu kugler) {
    this.kugler = kugler;
  }
  public void actionPerformed(ActionEvent e) {
    kugler.farve[kugler.antal-1]=0;
  }
}

class Roed implements ActionListener {
  KugleAnimationMenu kugler;
  Roed(KugleAnimationMenu kugler) {
    this.kugler = kugler;
  }

  public void actionPerformed(ActionEvent e) {
    kugler.farve[kugler.antal-1]=1;
  }
}

class Groen implements ActionListener {
  KugleAnimationMenu kugler;
  Groen(KugleAnimationMenu kugler) {
    this.kugler = kugler;
  }
  public void actionPerformed(ActionEvent e) {
    kugler.farve[kugler.antal-1]=2;
  }
}

class Blaa implements ActionListener {
  KugleAnimationMenu kugler;
  Blaa(KugleAnimationMenu kugler) {
    this.kugler = kugler;
  }
  public void actionPerformed(ActionEvent e) {
    kugler.farve[kugler.antal-1]=3;
  }
}

class Hvid implements ActionListener {
  KugleAnimationMenu kugler;
  Hvid(KugleAnimationMenu kugler) {
    this.kugler = kugler;
  }
  public void actionPerformed(ActionEvent e) {
    kugler.farve[kugler.antal-1]=4;
  }
}

class MainMenuListener implements MenuListener {
  Timer timer;
  MainMenuListener(Timer timer) {
    this.timer = timer;
  }
  public void menuCanceled(MenuEvent e) {
    timer.restart();
  }
  public void menuDeselected(MenuEvent e) {
    timer.restart();
  }
  public void menuSelected(MenuEvent e) {
    timer.stop();
  }
}

class DrawingCanvas extends JComponent {
  int width,height;

  DrawingCanvas(int width, int height) {
    this.width = width;
    this.height = height;
  }

  private Image offscreenImage;

  public void paint(Graphics screen) {
    if (offscreenImage!=null) screen.drawImage(offscreenImage,0,0,width,height,null);
  }

  public Image getOffscreenImage() {
    if (offscreenImage==null) offscreenImage = createImage(width,height);
    return offscreenImage;
  }

  public Dimension getMinimumSize() {
    return new Dimension(width,height);
  }

  public Dimension getPreferredSize() {
      return new Dimension(width,height);
  }
}
